@@ -5,31 +5,26 @@
 GenIce class
 """
 
-from logging import getLogger
-from collections import defaultdict
-import random
 import itertools as it
-import string
+import random
+from collections import defaultdict
+from logging import getLogger
 from types import SimpleNamespace
 
 import numpy as np
 import pairlist as pl
-from cycless.polyhed import polyhedra_iter, cage_to_graph
-from cycless.cycles import centerOfMass, cycles_iter
 import tilecycles as tc
 
-from genice2 import digraph as dg
-from genice2.plugin import safe_import
-from genice2.cell import rel_wrap, Cell
-from genice2.valueparser import parse_cages, parse_pairs, put_in_array, flatten, plugin_option_parser
-from genice2.decorators import timeit, banner
 # for alkyl groups (Experimental)
-from genice2 import alkyl
+from genice2 import alkyl, cage
+from genice2 import digraph as dg
+from genice2.cell import Cell, rel_wrap
+from genice2.decorators import banner, timeit
 # A virtual monatomic molecule
-from genice2.molecules import one, arrange, monatom
-
-# for cage assessment
-from graphstat import GraphStat
+from genice2.molecules import arrange, monatom, one
+from genice2.plugin import safe_import
+from genice2.valueparser import (flatten, parse_cages, parse_pairs,
+                                 plugin_option_parser, put_in_array)
 
 
 def assume_tetrahedral_vectors(v):
@@ -434,7 +429,7 @@ class GenIce():
                  spot_groups=dict(),
                  asis=False,
                  shift=(0., 0., 0.),
-                 seed=1000,
+                 # seed=1000,
                  ):
         """
         Constructor of GenIce.
@@ -450,7 +445,6 @@ class GenIce():
             spot_groups:Labels of cages in which a group is placed.
             asis:       Do not modify the orientations of the hydrogen bonds.
             shift:      A fractional value to be added to the positions.
-            seed:       A random seed.
 
         """
         logger = getLogger()
@@ -460,13 +454,8 @@ class GenIce():
         self.anions = anions
         self.spot_guests = spot_guests
         self.spot_groups = spot_groups
+
         # Show the document of the module
-
-        # Set random seeds
-        random.seed(seed)
-        np.random.seed(seed)
-        self.seed = seed  # used in tilecycles
-
         try:
             self.doc = lat.__doc__.splitlines()
         except BaseException:
@@ -680,19 +669,19 @@ class GenIce():
             self.Stage2()
 
             # Count bonds
-            nrandom = 0
+            num_hb_disorder = 0
             nfixed = 0
             for i, j, data in self.graph.edges(data=True):
                 if self.graph[i][j]['fixed']:  # fixed pair
                     nfixed += 1
                 else:
-                    nrandom += 1
+                    num_hb_disorder += 1
             logger.info(
                 f"  Number of pre-oriented hydrogen bonds: {nfixed}")
             logger.info(
-                f"  Number of unoriented hydrogen bonds: {nrandom}")
+                f"  Number of unoriented hydrogen bonds: {num_hb_disorder}")
             logger.info("  Number of hydrogen bonds: {0} (regular num: {1})".format(
-                nfixed + nrandom, len(self.reppositions) * 2))
+                nfixed + num_hb_disorder, len(self.reppositions) * 2))
 
             # test2==True means it is a z=4 graph.
             test2 = self.test_undirected_graph(self.graph)
@@ -704,15 +693,15 @@ class GenIce():
                 if maxstage < 3 or abort:
                     return
 
-            # cyclefiller == fast algorithm
-            cyclefiller = True
+            # new_algorithm == fast algorithm
+            new_algorithm = True
             # it makes the digraph obeying ice rule with zero net polarization
             # but it works only for a perfect 4-graph.
             if not test2 or self.asis or nfixed > 0 or depol != "strict":
                 # The network is not 4-connected.
-                cyclefiller = False
+                new_algorithm = False
 
-            if cyclefiller:
+            if new_algorithm:
                 # Fast track
                 self.Stage3D()
             else:
@@ -727,7 +716,14 @@ class GenIce():
             # spacegraph might be already set in Stage3D.
             if self.spacegraph is None:
                 logger.debug(f"  graph? {self.spacegraph}")
-                self.Stage4(depol=depol)
+                if num_hb_disorder == 0:
+                    self.Stage4(depol="none")
+                else:
+                    self.Stage4(depol=depol)
+
+            dipole = self.spacegraph.net_polarization()
+            logger.info(f"Residual net polarization: {dipole[0]:.2f} {dipole[1]:.2f} {dipole[2]:.2f}")
+
 
             if 4 in hooks:
                 abort = hooks[4](self)
@@ -787,63 +783,18 @@ class GenIce():
 
         if noise > 0.0:
             logger.info(f"  Add noise: {noise}.")
-            perturb = np.random.normal(loc=0.0,
-                                       scale=noise * 0.01 * 3.0 * 0.5,  # in percent, radius of water
-                                       size=self.reppositions.shape)
+            perturb = np.random.normal(
+                loc=0.0,
+                scale=noise * 0.01 * 3.0 * 0.5,  # in percent, radius of water
+                size=self.reppositions.shape)
             self.reppositions += self.repcell.abs2rel(perturb)
 
         if assess_cages:
-            import networkx as nx
             logger.info("  Assessing the cages...")
-            # Prepare the list of rings
-            ringlist = [[int(x) for x in ring] for ring in cycles_iter(
-                nx.Graph(self.graph), 8, pos=self.waters)]
-            # Positions of the centers of the rings.
-            ringpos = [centerOfMass(ringnodes, self.waters)
-                       for ringnodes in ringlist]
-            maxcagesize = 22
-            cagepos = []
-            cagetypes = []
-            # Detect cages and classify
-            db = GraphStat()
-            labels = set()
-            g_id2label = dict()
-            for cage in polyhedra_iter(ringlist, maxcagesize):
-                cagepos.append(centerOfMass(list(cage), ringpos))
-                g = cage_to_graph(cage, ringlist)
-                cagesize = len(cage)
-                g_id = db.query_id(g)
-                if g_id < 0:
-                    g_id = db.register()
-                    enum = 0
-                    label = f"A{cagesize}"
-                    while label in labels:
-                        char = string.ascii_lowercase[enum]
-                        label = f"A{cagesize}{char}"
-                        enum += 1
-                    g_id2label[g_id] = label
-                    labels.add(label)
-                    ringcount = [0 for i in range(9)]
-                    for ring in cage:
-                        ringcount[len(ringlist[ring])] += 1
-                    index = []
-                    for i in range(9):
-                        if ringcount[i] > 0:
-                            index.append(f"{i}^{ringcount[i]}")
-                    index = " ".join(index)
-                    logger.info(f"    Cage type: {label} ({index})")
-
-                else:
-                    label = g_id2label[g_id]
-                cagetypes.append(label)
-            if len(cagepos) > 0:
-                self.cagepos = np.array(cagepos)
-                self.cagetype = cagetypes
-            else:
-                logger.info("    No cages detected.")
+            self.cagepos, self.cagetype = cage.assess_cages( self.graph, self.waters )
             logger.info("  Done assessment.")
 
-        if self.cagepos is not None:
+        if self.cagepos is not None and self.cagepos.shape[0] > 0:
             logger.info("  Hints:")
             self.repcagepos = replicate_positions(self.cagepos, self.rep)
             nrepcages = self.repcagepos.shape[0]
